/*------------------------------------------------------------------------------

 Author:  IAP GmbH
 Purpose: Demo code for OOABL pattern presentation
          This userbuilder object demonstrates the Builder Pattern

 Notes:
 - This class uses itself as return value, so the class itself must be in 'using'
 - Properties Setter are private, so setter methods must be used
 - Most methods return the current object (the builder)
 - The build method creates the user object and returns the reference.
   In the user object the properties of the builder are accessed, to create the user.

------------------------------------------------------------------------------*/

USING 01_builder.User.
USING 01_builder.UserBuilder.

CLASS 01_builder.UserBuilder:

/*---------------------------------Definitions----------------------------------*/

  /* required */
  DEFINE PUBLIC PROPERTY cFirstName AS CHARACTER NO-UNDO
    PUBLIC GET.
    PRIVATE SET.
  DEFINE PUBLIC PROPERTY cLastName AS CHARACTER NO-UNDO
    PUBLIC GET.
    PRIVATE SET.

  /* optional */
  DEFINE PUBLIC PROPERTY iAge AS INTEGER NO-UNDO 
    PUBLIC GET.
    PRIVATE SET.
  DEFINE PUBLIC PROPERTY cPhone AS CHARACTER NO-UNDO
    PUBLIC GET.
    PRIVATE SET.
  DEFINE PUBLIC PROPERTY cAddress AS CHARACTER NO-UNDO
    PUBLIC GET.
    PRIVATE SET.


/*-------------------------------- Constructor----------------------------------*/

  CONSTRUCTOR PUBLIC UserBuilder(cFirstName AS CHARACTER, cLastName AS CHARACTER):
    THIS-OBJECT:cFirstName = cFirstName.
    THIS-OBJECT:cLastName  = cLastName.
  END CONSTRUCTOR.

/* Setter Methods */
  METHOD PUBLIC UserBuilder setAge(iAge AS INTEGER):
    THIS-OBJECT:iAge = iAge.
    RETURN THIS-OBJECT.
  END METHOD.
  METHOD PUBLIC UserBuilder setPhone(cPhone AS CHARACTER):
    THIS-OBJECT:cPhone = cPhone.
    RETURN THIS-OBJECT.
  END METHOD.
  METHOD PUBLIC UserBuilder setAddress(cAddress AS CHARACTER):
    THIS-OBJECT:cAddress = cAddress.
    RETURN THIS-OBJECT.
  END METHOD.

  // Build method, give builder to user object and return user
  METHOD PUBLIC User build():
    RETURN NEW User(THIS-OBJECT).
  END METHOD.

END CLASS.
