
 /*------------------------------------------------------------------------
    File        : 02_singleton/Configuration
    Purpose     : Example for the Singleton Design Pattern
    Author(s)   : Michael Barfs, Xara Arnold, Klaus Erichsen

  This object is a configuration holder. In practice it will have many
  properties and will load the properties from DB, and/or file and/or 
  from OS and so on.
  The properties will be set inside this object and can be requested from
  ----------------------------------------------------------------------*/

USING Progress.Lang.*.
USING 02_singleton.Configuration.

CLASS 02_singleton.Configuration:

  DEFINE PROPERTY cBaseExportPath AS CHARACTER PUBLIC GET. PRIVATE SET.
  DEFINE PROPERTY iTenant         AS INTEGER   PUBLIC GET. PRIVATE SET.
  // many more properties...
   

  /* This _static_ propery is the key, it will return the only
     dynamic instance of this object. First call will create it. */
  DEFINE PUBLIC STATIC PROPERTY oInstance AS Configuration NO-UNDO
    PUBLIC GET():
      IF NOT VALID-OBJECT(oInstance) THEN
        oInstance = NEW Configuration().
      RETURN oInstance.
    END GET.
    PRIVATE SET.

  CONSTRUCTOR PRIVATE Configuration():
    // Load config from file, OS, DB, ...
    cBaseExportPath = "d:\dev\exports".
    iTenant = 435.
  END CONSTRUCTOR.

END CLASS.